@@ -47,7 +47,7 @@ QString TabWidget::currentState() const
 
 bool TabWidget::isKommanderWidget() const
 {
-  return TRUE;
+  return true;
 }
 
 QStringList TabWidget::associatedText() const
@@ -94,7 +94,7 @@ QString TabWidget::handleDCOP(int function, const QStringList& args)
       setCurrentPage(args[0].toUInt());
       break;
     default:
-      break;
+      return KommanderWidget::handleDCOP(function, args);
   }
   return QString::null;
 }
