@@ -43,6 +43,7 @@
 #include "debuggerui.h"
 #include "debuggervariable.h"
 #include "pathmapper.h"
+#include "variableslistview.h"
 #include "conditionalbreakpointdialog.h"
 
 // dialogs
@@ -65,11 +66,17 @@ void DebuggerManager::slotNewProjectLoaded(const QString &projectname, const KUR
 {
   if(m_client)
   {
+    
+    disconnect(m_client, SIGNAL(updateStatus(DebuggerUI::DebuggerStatus)), m_debuggerui, SLOT(slotStatus(DebuggerUI::DebuggerStatus)));
+
     delete m_client;
     m_client = NULL;
   }
   enableAction("*", false);
-
+  
+  // Remove all breakpoints
+  m_breakpointList->clear();
+  
   if(m_debuggerui)
   {
     delete m_debuggerui;
@@ -89,7 +96,12 @@ void DebuggerManager::slotNewProjectLoaded(const QString &projectname, const KUR
       if(Project::ref()->debuggerClient() == service->name())
       {
         int errCode = 0;
-        m_client = KParts::ComponentFactory::createInstanceFromService<DebuggerClient::DebuggerClient>(service, this, 0, QStringList(), &errCode);
+//Workaround for dynamic_cast not working correctly on SUSE 10, gcc 4.0.2
+//The correct way should be a simple:
+// m_client = KParts::ComponentFactory::createInstanceFromService<DebuggerClient>(service, this, 0, QStringList(), &errCode);
+        QObject* obj = KParts::ComponentFactory::createInstanceFromService<QObject>(service, this, 0, QStringList(), &errCode);
+        if (obj && obj->inherits("DebuggerClient"))
+          m_client = static_cast<DebuggerClient *>(obj);
 
         //kdDebug(24002) << service->name() << " (" << m_client << ")" << endl;
 
@@ -104,38 +116,92 @@ void DebuggerManager::slotNewProjectLoaded(const QString &projectname, const KUR
   }
 
   // Tell client to load its settings
-  if(m_client)
+  if (m_client)
   {
     QDomNode nodeThisDbg;
-    QDomNode projectNode = Project::ref()->dom()->firstChild().firstChild();
+    QDomDocument *dom = Project::ref()->sessionDom();
+    QDomNode projectNode = dom->firstChild().firstChild();
     QDomNode nodeDbg  = projectNode.namedItem("debuggers");
     if(nodeDbg.isNull())
     {
-      nodeDbg = Project::ref()->dom()->createElement("debuggers");
+      nodeDbg = dom->createElement("debuggers");
       projectNode.appendChild(nodeDbg);
     }
 
     // Load this project's mapped paths
     m_pathmapper->readConfig();
-
+    
     // Load this projects debugger's settings
     nodeThisDbg = nodeDbg.namedItem(m_client->getName());
     if(nodeThisDbg.isNull())
     {
-      nodeThisDbg = Project::ref()->dom()->createElement(m_client->getName());
+      nodeThisDbg = dom->createElement(m_client->getName());
       nodeDbg.appendChild(nodeThisDbg);
     }
 
     m_client->readConfig(nodeThisDbg);
-    
+
+    // recreate UI
     m_debuggerui = new DebuggerUI(this, "debuggerui");
+    connect(m_client, SIGNAL(updateStatus(DebuggerUI::DebuggerStatus)), m_debuggerui, SLOT(slotStatus(DebuggerUI::DebuggerStatus)));
+    
+    // Load saved breakpoints
+    if(Project::ref()->debuggerPersistentBreakpoints())
+    {
+      QDomNode nodeBreakpoints = nodeDbg.namedItem("breakpoints");
+      if(!nodeBreakpoints.isNull())
+      {
+        QDomNode child = nodeBreakpoints.firstChild();
+        while(!child.isNull())
+        {
+          DebuggerBreakpoint* bp = new DebuggerBreakpoint();
+          bp->setFilePath( child.attributes().namedItem("filepath").nodeValue());
+          bp->setClass( child.attributes().namedItem("class").nodeValue());
+          bp->setFunction( child.attributes().namedItem("function").nodeValue());
+          bp->setCondition( child.attributes().namedItem("condition").nodeValue());
+          bp->setLine( child.attributes().namedItem("line").nodeValue().toLong());
+          if(child.attributes().namedItem("type").nodeValue() == "true")
+            bp->setType(DebuggerBreakpoint::ConditionalTrue);
+          else if(child.attributes().namedItem("type").nodeValue() == "change")
+            bp->setType(DebuggerBreakpoint::ConditionalChange);
+          else
+            bp->setType(DebuggerBreakpoint::LineBreakpoint);
+          
+          // Update client and ui
+          m_client->addBreakpoint(bp);
+          m_breakpointList->add(bp);
+          
+          // loop
+          child = child.nextSibling();
+        }
+      }
+    }
+
+    // Load saved Watches
+    if(Project::ref()->debuggerPersistentWatches())
+    {
+      QDomNode nodeWatches = nodeDbg.namedItem("watches");
+      if(!nodeWatches.isNull())
+      {
+        QDomNode child = nodeWatches.firstChild();
+        while(!child.isNull())
+        {
+          QString watch = child.attributes().namedItem("name").nodeValue();
+          DebuggerVariable *var = new DebuggerVariable(watch, "", DebuggerVariableTypes::Undefined);
+          m_debuggerui->addVariable(var);
+          m_client->addWatch(watch);
+
+          child = child.nextSibling();
+        }
+      }
+    }
+
   }
 
   initClientActions();
 
   // Disable all debugactions that need a session (ie not breakpoints, etc)
   slotDebugStartSession();
-
 }
 
 void DebuggerManager::initActions()
@@ -162,11 +228,11 @@ void DebuggerManager::initActions()
   newaction = new KAction(i18n("Send HTTP R&equest"), SmallIcon("debug_currentline"), 0, this, SLOT(slotDebugRequest()), ac, "debug_request");
   newaction->setToolTip(i18n("Initiate HTTP Request to the server with debugging activated"));
   
-  newaction = new KAction(i18n("&Run"), SmallIcon("debug_run"), 0, this, SLOT(slotDebugRun()), ac, "debug_run");
-  newaction->setToolTip(i18n("Executes the script showing every line of execution to the user (slow). If a script is currently not being debugged, the next script started will start in run mode"));
+  newaction = new KAction(i18n("&Trace"), SmallIcon("debug_run"), 0, this, SLOT(slotDebugTrace()), ac, "debug_trace");
+  newaction->setToolTip(i18n("Traces through the script. If a script is currently not being debugged, it will start in trace mode when started"));
   
-  newaction = new KAction(i18n("&Leap"), SmallIcon("debug_leap"), 0, this, SLOT(slotDebugLeap()), ac, "debug_leap");
-  newaction->setToolTip(i18n("Executes the script without showing every line of execution to the user (fast). If a script is currently not being debugged, the next script started will start in leap mode"));
+  newaction = new KAction(i18n("&Run"), SmallIcon("debug_leap"), 0, this, SLOT(slotDebugRun()), ac, "debug_run");
+  newaction->setToolTip(i18n("Runs the script. If a script is currently not being debugged, it will start in run mode when started"));
   
   newaction = new KAction(i18n("&Step"), SmallIcon("debug_stepover"), 0, this, SLOT(slotDebugStepOver()), ac, "debug_stepover");
   newaction->setToolTip(i18n("Executes the next line of execution, but does not step into functions or includes"));
@@ -181,7 +247,7 @@ void DebuggerManager::initActions()
   newaction->setToolTip(i18n("Executes the rest of the commands in the current function/file and pauses when it is done (when it reaches a higher level in the backtrace)"));
   
   newaction = new KAction(i18n("&Pause"), SmallIcon("debug_pause"), 0, this, SLOT(slotDebugPause()), ac, "debug_pause");
-  newaction->setToolTip(i18n("Pauses the scripts if it is running or leaping. If a script is currently not being debugged, it will start in paused mode when started"));
+  newaction->setToolTip(i18n("Pauses the scripts if it is running or tracing. If a script is currently not being debugged, it will start in paused mode when started"));
   newaction = new KAction(i18n("Kill"), SmallIcon("debug_kill"), 0, this, SLOT(slotDebugKill()), ac, "debug_kill");
   newaction->setToolTip(i18n("Kills the currently running script"));
   
@@ -204,6 +270,9 @@ void DebuggerManager::initActions()
   newaction = new KAction(i18n("Set Value of Variable"), SmallIcon("edit"), 0, this, SLOT(slotVariableSet()), ac, "debug_variable_setdialog");
   newaction->setToolTip(i18n("Changes the value of a variable"));
 
+  newaction = new KAction(i18n("Open Profiler Output"), SmallIcon("launch"), 0, this, SLOT(slotProfilerOpen()), ac, "debug_profiler_open");
+  newaction->setToolTip(i18n("Opens the profiler output file"));
+
   enableAction("*", false);
 
 }
@@ -226,8 +295,16 @@ DebuggerManager::~DebuggerManager()
 {
   delete m_breakpointList;
   m_breakpointList = 0L;
-  delete m_client;
-  m_client = 0L;
+
+  if(m_client)
+  {
+        
+    disconnect(m_client, SIGNAL(updateStatus(DebuggerUI::DebuggerStatus)), m_debuggerui, SLOT(slotStatus(DebuggerUI::DebuggerStatus)));
+
+    delete m_client;
+    m_client = 0L;
+  }
+
   delete m_debuggerui;
   m_debuggerui = 0L;
   delete m_interface;
@@ -243,7 +320,7 @@ void DebuggerManager::enableAction(const QString& action, bool enable)
     // Enable/Disable all session related actions + connect/disconnect
     enableAction("debug_request", enable);
     enableAction("debug_run", enable);
-    enableAction("debug_leap", enable);
+    enableAction("debug_trace", enable);
     enableAction("debug_pause", enable);
     enableAction("debug_kill", enable);
     enableAction("debug_stepover", enable);
@@ -256,6 +333,8 @@ void DebuggerManager::enableAction(const QString& action, bool enable)
 
     enableAction("debug_breakpoints_toggle", enable);
     enableAction("debug_breakpoints_clear", enable);
+    
+    enableAction("debug_profiler_open", enable);
 
   }
   else
@@ -278,7 +357,7 @@ void DebuggerManager::slotRemoveVariable(DebuggerVariable* var)
 
 void DebuggerManager::slotRemoveBreakpoint(DebuggerBreakpoint* bp)
 {
-  if(!m_client || !bp)
+  if(!m_client)
     return;
   m_breakpointList->remove(bp);
   m_client->removeBreakpoint(bp);
@@ -345,15 +424,6 @@ void DebuggerManager::slotConditionalBreakpoint()
       m_breakpointList->add(bp);
     }
   }
-    
-  /*QString condition = KInputDialog::getText(i18n("Add Conditional Breakpoint"), i18n("Specify expression to break at (when true):"), quantaApp->popupWord);
-  quantaApp->popupWord = "";
-  if(!condition.isEmpty())
-  {
-    DebuggerBreakpoint * bp = new DebuggerBreakpoint("", 0, condition);
-    m_client->addBreakpoint(bp);
-    m_breakpointList->add(bp);
-  }*/
 }
 
 void DebuggerManager::slotDebugStartSession()
@@ -380,20 +450,20 @@ void DebuggerManager::slotDebugRequest()
   m_client->request();
 }
 
+void DebuggerManager::slotDebugTrace()
+{
+  if(!m_client)
+    return;
+
+  m_client->trace();
+}
+
 void DebuggerManager::slotDebugRun()
 {
   if(!m_client)
     return;
 
   m_client->run();
-}
-
-void DebuggerManager::slotDebugLeap()
-{
-  if(!m_client)
-    return;
-
-  m_client->leap();
 
 }
 void DebuggerManager::slotDebugSkip()
@@ -445,6 +515,13 @@ void DebuggerManager::slotDebugStepOut()
 
 }
 
+void DebuggerManager::slotProfilerOpen( )
+{
+  if(!m_client)
+    return;
+
+  m_client->profilerOpen();
+}
 
 // A new file was opened, tell the debugger so it can tell us about breakpoints etc
 void DebuggerManager::fileOpened(const QString& file)
@@ -461,6 +538,17 @@ void DebuggerManager::fileOpened(const QString& file)
     }
   }
 
+  //lets keep the eye on toggling bp's through the editor margin
+  QuantaView *view = ViewManager::ref()->isOpened(KURL::fromPathOrURL(file));
+  if (view)
+  {
+    ::Document* qdoc = view->document();
+    if(qdoc)
+    {
+      connectBreakpointSignals(qdoc);
+    }
+  }
+
   // Also, if we have a debug-session, let the debugger know...
   if(m_client)
     m_client->fileOpened(file);
@@ -469,9 +557,6 @@ void DebuggerManager::fileOpened(const QString& file)
 // Check with editors if breakpoints changed and send all breakpoint (again) to client
 void DebuggerManager::refreshBreakpoints()
 {
-  // Update bp-list from editors
-  // ...TODO
-
   // Resend bps
   m_breakpointList->rewind();
   DebuggerBreakpoint* bp;
@@ -516,7 +601,7 @@ bool DebuggerManager::setActiveLine (const QString& file, int line )
     return true;
 
   // Find new position in editor
-  if(ViewManager::ref()->isOpened(filename) || QExtFileInfo::exists(filename))
+  if(ViewManager::ref()->isOpened(filename) || QExtFileInfo::exists(filename, 0L))
     quantaApp->gotoFileAndLine(filename, line, 0);
   else
   {
@@ -536,6 +621,8 @@ void DebuggerManager::setMark(const QString& filename, long line, bool set, int
     ::Document* qdoc = ViewManager::ref()->isOpened(filename)->document();
     if(qdoc)
     {
+      disconnectBreakpointSignals(qdoc);
+
       KTextEditor::Document* doc = qdoc->doc();
       if(doc)
       {
@@ -548,10 +635,29 @@ void DebuggerManager::setMark(const QString& filename, long line, bool set, int
             markIf->removeMark(line, mark);
         }
       }
+      connectBreakpointSignals(qdoc);
     }
   }
 }
 
+void DebuggerManager::connectBreakpointSignals(Document* qdoc)
+{
+  connect(qdoc, SIGNAL(breakpointMarked(Document*, int)),
+    this, SLOT(slotBreakpointMarked(Document*, int)));
+
+  connect(qdoc, SIGNAL(breakpointUnmarked(Document*, int)),
+    this, SLOT(slotBreakpointUnmarked(Document*, int)));
+}
+
+void DebuggerManager::disconnectBreakpointSignals(Document* qdoc)
+{
+  disconnect(qdoc, SIGNAL(breakpointMarked(Document*, int)),
+    this, SLOT(slotBreakpointMarked(Document*, int)));
+
+  disconnect(qdoc, SIGNAL(breakpointUnmarked(Document*, int)),
+    this, SLOT(slotBreakpointUnmarked(Document*, int)));
+}
+
 // Show a status message and optionally put it on the log
 bool DebuggerManager::showStatus(const QString& a_message, bool log)
 {
@@ -576,15 +682,18 @@ void DebuggerManager::toggleBreakpoint ()
     uint line, col;
     w->viewCursorIf->cursorPositionReal(&line, &col);
 
-    DebuggerBreakpoint* br = new DebuggerBreakpoint(w->url().prettyURL(0, KURL::StripFileProtocol), line);
+    DebuggerBreakpoint* br = m_breakpointList->retrieve(w->url().prettyURL(0, KURL::StripFileProtocol), line);
 
-    if(!m_breakpointList->exists(br))
+    if(!br)
     {
+      DebuggerBreakpoint* br = new DebuggerBreakpoint(w->url().prettyURL(0, KURL::StripFileProtocol), line);
       m_breakpointList->add(br);
-      setMark(w->url().prettyURL(0, KURL::StripFileProtocol), br->line(), true, KTextEditor::MarkInterface::markType02);
+//       setMark(w->url().prettyURL(0, KURL::StripFileProtocol), br->line(), true, KTextEditor::MarkInterface::markType02); // FIXME Is this really needed?
       if(m_client && m_client->isActive())
       {
+        DebuggerBreakpoint tmpbp = *br;
         m_client->addBreakpoint(br);
+        
       }
       else
         // Trigger pathmapper to make sure we have a valid translation...
@@ -593,7 +702,7 @@ void DebuggerManager::toggleBreakpoint ()
     else
     {
       m_breakpointList->remove(br);
-      setMark(w->url().prettyURL(0, KURL::StripFileProtocol), br->line(), false, KTextEditor::MarkInterface::markType02);
+//       setMark(w->url().prettyURL(0, KURL::StripFileProtocol), br->line(), false, KTextEditor::MarkInterface::markType02); // FIXME Is this really needed?
 
       if(m_client && m_client->isActive())
       {
@@ -607,9 +716,135 @@ void DebuggerManager::clearBreakpoints ()
   m_breakpointList->clear();
 }
 
-DebuggerBreakpoint *DebuggerManager::newDebuggerBreakpoint()
+void DebuggerManager::slotBreakpointMarked(Document* qdoc, int line)
 {
-  return new DebuggerBreakpoint();
+  DebuggerBreakpoint* br = new DebuggerBreakpoint(qdoc->url().prettyURL(0, KURL::StripFileProtocol), line);
+
+  m_breakpointList->add(br);
+  if(m_client && m_client->isActive())
+  {
+    m_client->addBreakpoint(br);
+  }
 }
 
+void DebuggerManager::slotBreakpointUnmarked(Document* qdoc, int line)
+{
+  QString filePath = qdoc->url().prettyURL(0, KURL::StripFileProtocol);
+
+  DebuggerBreakpoint* br = m_breakpointList->retrieve(filePath, line);
+
+  if (br)
+  {
+    if(m_client && m_client->isActive())
+    {
+      m_client->removeBreakpoint(br);
+    }
+  
+    m_breakpointList->remove(br);
+  }
+}
+  
+void DebuggerManager::updateBreakpointKey( const DebuggerBreakpoint & bp, const QString & newkey )
+{
+  m_breakpointList->updateBreakpointKey(bp, newkey);
+ 
+  // Update UI
+  m_debuggerui->deleteBreakpoint(bp);
+  DebuggerBreakpoint bpnew(bp);
+  bpnew.setKey(newkey);
+  m_debuggerui->showBreakpoint(bpnew);
+ 
+}
+
+DebuggerBreakpoint * DebuggerManager::findDebuggerBreakpoint( const QString & key )
+{
+  return m_breakpointList->findDebuggerBreakpoint(key);
+}
+
+void DebuggerManager::saveProperties( )
+{
+  
+  if (m_client)
+  {
+    QDomDocument *dom = Project::ref()->sessionDom();
+    QDomNode projectNode = dom->firstChild().firstChild();
+    QDomNode nodeDbg  = projectNode.namedItem("debuggers");
+    if(nodeDbg.isNull())
+    {
+      nodeDbg = dom->createElement("debuggers");
+      projectNode.appendChild(nodeDbg);
+    }
+
+    // Save breakpoints
+    if(Project::ref()->debuggerPersistentBreakpoints())
+    {
+      // (Re)create breakpoints section
+      QDomNode nodeBreakpoints = nodeDbg.namedItem("breakpoints");
+      if(!nodeBreakpoints.isNull())
+        nodeBreakpoints.parentNode().removeChild(nodeBreakpoints);
+      
+      if(m_breakpointList->count() > 0)
+      {
+        nodeBreakpoints = dom->createElement("breakpoints");
+        nodeDbg.appendChild(nodeBreakpoints);
+      
+  
+        // Loop breakpoints and save 'em
+        m_breakpointList->rewind();
+        DebuggerBreakpoint* bp;
+        while((bp = m_breakpointList->next()))
+        {
+          QDomElement child = dom->createElement("breakpoint");
+          child.setAttribute("filepath", bp->filePath());
+          child.setAttribute("class", bp->inClass());
+          child.setAttribute("function", bp->inFunction());
+          child.setAttribute("condition", bp->condition());
+          child.setAttribute("line", QString::number(bp->line()));
+          if(bp->type() == DebuggerBreakpoint::ConditionalTrue)
+            child.setAttribute("type", "true");
+          else if(bp->type() == DebuggerBreakpoint::ConditionalChange)
+            child.setAttribute("type", "change");
+          else
+            child.setAttribute("type", "line");
+          
+          nodeBreakpoints.appendChild(child);
+        }
+      }
+    }
+
+    // Save Watches
+    if(Project::ref()->debuggerPersistentWatches())
+    {
+      // (Re)create watches section
+      QDomNode nodeWatches = nodeDbg.namedItem("watches");
+      if(!nodeWatches.isNull())
+        nodeWatches.parentNode().removeChild(nodeWatches);
+      
+      if(m_debuggerui->watches()->first())
+      {
+        nodeWatches = dom->createElement("watches");
+        nodeDbg.appendChild(nodeWatches);
+        
+        // Loop watches and save 'em
+        for( DebuggerVariable *v = m_debuggerui->watches()->first(); v; v = m_debuggerui->watches()->next())
+        {
+          QDomElement child = dom->createElement("watch");
+          child.setAttribute("name", v->name());
+          
+          nodeWatches.appendChild(child);
+        }
+      }
+    }
+  }
+}
+
+void DebuggerManager::slotHandleEvent( const QString & event, const QString &, const QString & )
+{
+  if(event == "before_project_close")
+    saveProperties();
+}
+
+
+
+
 #include "debuggermanager.moc"
