@@ -49,6 +49,7 @@
 #include "wkafkapart.h"
 #include "undoredo.h"
 #include "nodeproperties.h"
+#include "tagaction.h"
 #include "tagactionset.h"
 #include "document.h"
 #include "resource.h"
@@ -84,7 +85,7 @@ public:
 
 KafkaWidget::KafkaWidget(QWidget *parent, QWidget *widgetParent, KafkaDocument *part,
                          const char *name)
-        : KHTMLPart(new KHTMLView(this, widgetParent, name), parent, name),
+        : KHTMLPart(widgetParent, name, parent, name),
         w(part)
 {
     m_contextPopupMenu = new QPopupMenu();
@@ -205,7 +206,11 @@ void KafkaWidget::insertText(DOM::Node node, const QString &text, int position)
     {
         DOM::Text textNode = document().createTextNode(text);
         DOM::Node parent = node.parentNode();
-        parent.insertBefore(textNode, node);
+//FIXME: Andras: safety checks, as parent  can be null. Maybe it just hides the error...
+        if (!parent.isNull())
+          parent.insertBefore(textNode, node);
+        else
+          node.appendChild(textNode);
         m_currentNode = textNode;
         d->m_cursorOffset = text.length();
         emit domNodeInserted(textNode, false, m_modifs);
@@ -219,7 +224,17 @@ void KafkaWidget::insertText(DOM::Node node, const QString &text, int position)
     {
         DOM::Text textNode = document().createTextNode(text);
         DOM::Node parent = node.parentNode();
-        parent.insertBefore(textNode, node.nextSibling());
+//FIXME: Andras: safety checks, as parent and node.nextSibling can be null. Maybe it just hides the error...
+//Also it seems that position can be 3 and node is "body". See bug 112733.
+        if (node.nodeName().string().lower() != "body" && !parent.isNull())
+        {
+          if (!node.nextSibling().isNull())
+            parent.insertBefore(textNode, node.nextSibling());
+          else
+            parent.insertBefore(textNode, node);
+        }
+        else
+          node.appendChild(textNode);
         m_currentNode = textNode;
         d->m_cursorOffset = text.length();
         emit domNodeInserted(textNode, false, m_modifs);
@@ -267,13 +282,14 @@ void KafkaWidget::normalize(DOM::Node _node)
     kdDebug(25001)<< "KafkaWidget::normalize()" << endl;
 #endif
 
+//FIXME: Andras: getAttrs() can sometimes return NULL and causes a crash. No idea why and what it means though.
     DOM::Node childNode = _node.firstChild();
     while(!childNode.isNull())
     {
-        if(w->getAttrs(childNode)->chCurFoc() == kNodeAttrs::textNode)
+        if(w->getAttrs(childNode) && w->getAttrs(childNode)->chCurFoc() == kNodeAttrs::textNode)
         {
             while(!childNode.nextSibling().isNull() &&
-                    w->getAttrs(childNode.nextSibling())->chCurFoc() ==
+                    w->getAttrs(childNode.nextSibling()) && w->getAttrs(childNode.nextSibling())->chCurFoc() ==
                     kNodeAttrs::textNode )
             {
                 childNode.setNodeValue(childNode.nodeValue() +
@@ -556,7 +572,7 @@ bool KafkaWidget::eventFilter(QObject *, QEvent *event)
         case Key_Escape:
             break;
         case Key_Tab:
-            if(!m_currentNode.isNull() &&
+            if(!m_currentNode.isNull() && w->getAttrs(m_currentNode) &&
                     w->getAttrs(m_currentNode)->chCurFoc() != kNodeAttrs::no)
             {
                 // @todo check tab settings in Quanta
@@ -578,6 +594,9 @@ bool KafkaWidget::eventFilter(QObject *, QEvent *event)
 #endif
             if(hasSelection())
                 removeSelection();
+                
+            applyQueuedToggableTagActions();                
+            
             keyReturn(keyevent->state() & ControlButton);
             d->stuckCursorHorizontalPos = false;
             break;
@@ -646,6 +665,8 @@ bool KafkaWidget::eventFilter(QObject *, QEvent *event)
                 {
                     if(hasSelection())
                         removeSelection();
+                
+                    applyQueuedToggableTagActions();                
 
                     insertText(keyevent->text(), -1);
                 }
@@ -833,7 +854,7 @@ void KafkaWidget::keyDelete()
             if(_node == 0)
                 break;
             attrs = w->getAttrs(_node);
-            if(attrs->chCurFoc() == kNodeAttrs::textNode)
+            if(attrs && attrs->chCurFoc() == kNodeAttrs::textNode)
             {
                 m_currentNode = _node;
                 d->m_cursorOffset =
@@ -843,7 +864,7 @@ void KafkaWidget::keyDelete()
                 emit domNodeNewCursorPos(m_currentNode, d->m_cursorOffset);
                 return;
             }
-            if(attrs->chCurFoc() != kNodeAttrs::no &&
+            if(attrs && attrs->chCurFoc() != kNodeAttrs::no &&
                     attrs->chCurFoc() != kNodeAttrs::textNode)
             {
                 m_currentNode = _node;
@@ -861,7 +882,7 @@ void KafkaWidget::keyDelete()
             if(_node == 0)
                 break;
             attrs = w->getAttrs(_node);
-            if(attrs->chCurFoc() != kNodeAttrs::no)
+            if(attrs && attrs->chCurFoc() != kNodeAttrs::no)
             {
                 m_currentNode = _node;
                 d->m_cursorOffset = 0;
@@ -897,7 +918,7 @@ void KafkaWidget::keyDelete()
         attrs = w->getAttrs(_nodeNext);
 
         //If this Node can't be deleted, we stop here.
-        if(!attrs->cbDel())
+        if(!attrs || !attrs->cbDel())
             return;
 
         //If we are in a TEXT node, we remove a letter
@@ -1141,7 +1162,7 @@ void KafkaWidget::keyDelete()
             attrs = w->getAttrs(_nodeNext);
 
             //If this Node can't be deleted, we stop here.
-            if(!attrs->cbDel())
+            if(!attrs || !attrs->cbDel())
                 break;
 
             //Let's delete useless Nodes
@@ -1164,7 +1185,7 @@ void KafkaWidget::keyDelete()
 
         //And finally, if the cursor is at a bad place (e.g. inside a Inline with childs), move it
         attrs = w->getAttrs(_node);
-        while(attrs->chCurFoc() == kNodeAttrs::inlineNode && _node.hasChildNodes())
+        while(attrs && attrs->chCurFoc() == kNodeAttrs::inlineNode && _node.hasChildNodes())
         {
             _node = kafkaCommon::getChildNode(_node, childPosition, true);
             childPosition = 1;
@@ -1252,7 +1273,7 @@ void KafkaWidget::keyBackspace()
             if(_node == 0)
                 break;
             attrs = w->getAttrs(_node);
-            if(attrs->chCurFoc() == kNodeAttrs::textNode)
+            if(attrs && attrs->chCurFoc() == kNodeAttrs::textNode)
             {
                 m_currentNode = _node;
                 d->m_cursorOffset = (static_cast<DOM::CharacterData>(_node)).length();
@@ -1261,7 +1282,7 @@ void KafkaWidget::keyBackspace()
                 emit domNodeNewCursorPos(m_currentNode, d->m_cursorOffset);
                 return;
             }
-            if(attrs->chCurFoc() != kNodeAttrs::no && attrs->chCurFoc() !=
+            if(attrs && attrs->chCurFoc() != kNodeAttrs::no && attrs->chCurFoc() !=
                     kNodeAttrs::textNode)
             {
                 m_currentNode = _node;
@@ -1280,7 +1301,7 @@ void KafkaWidget::keyBackspace()
             if(_node == 0)
                 break;
             attrs = w->getAttrs(_node);
-            if(attrs->chCurFoc() != kNodeAttrs::no)
+            if(attrs && attrs->chCurFoc() != kNodeAttrs::no)
             {
                 m_currentNode = _node;
                 d->m_cursorOffset = 0;
@@ -1787,6 +1808,60 @@ DOM::Node KafkaWidget::getPrevNode(DOM::Node _node, bool &goingTowardsRootNode,
     return 0;
 }
 
+void KafkaWidget::updateToggableTagActions(/*const DOM::Node &domNode, long offset*/) const
+{
+    quantaApp->removeAllTagActionPoolItems();
+    
+    NodeSelectionInd selection;
+    selection.fillWithVPLCursorSelection();
+    
+    Node* start_node = 0, *end_node = 0;
+//     int start_offset = 0, end_offset = 0;
+        
+    start_node = kafkaCommon::getNodeFromLocation(selection.cursorNode());
+//     start_offset = selection.cursorOffset();
+    
+    if(!start_node)
+        return;
+    
+    if(selection.hasSelection())
+    {
+        end_node = kafkaCommon::getNodeFromLocation(selection.cursorNodeEndSel());
+//         end_offset = selection.cursorOffsetEndSel();
+    }
+    else
+    {
+        end_node = start_node;
+//         end_offset = start_offset;
+    }
+    
+    // Iterate all toggable toolbar actions and toggle them on or off
+    // Look if there is a selection
+    TagAction* tag_action = 0;    
+    QPtrList<TagAction> tag_actions = quantaApp->tagActions();        
+    for (tag_action = tag_actions.first(); tag_action; tag_action = tag_actions.next())
+    {
+        if(tag_action->toggable())
+        {
+            QString tag_name = tag_action->XMLTagName();
+            if(tag_name.isEmpty())
+                break;
+            
+            QDomElement data(tag_action->data());
+            QString attribute_name(data.attribute("attribute_name", QString()));
+            QString attribute_value(data.attribute("attribute_value", QString()));
+            
+            int inside_tag;
+            if(!attribute_name.isEmpty() && !attribute_value.isEmpty())
+                inside_tag = kafkaCommon::isInsideTag(start_node, end_node, tag_name, attribute_name, attribute_value);
+            else
+                inside_tag = kafkaCommon::isInsideTag(start_node, end_node, tag_name);
+            
+            tag_action->setChecked(inside_tag == 1);
+        }
+    }
+}
+
 void KafkaWidget::makeCursorVisible(int , int )
 {
     /**DOM::Range range;
@@ -1843,7 +1918,7 @@ void KafkaWidget::postprocessCursorPosition()
              if(_nextNode.isNull())
                  break;
              attrs2 = w->getAttrs(_nextNode);
-             if(attrs2->chCurFoc() == kNodeAttrs::textNode &&
+             if(attrs2 && attrs2->chCurFoc() == kNodeAttrs::textNode &&
                      (static_cast<DOM::CharacterData>(_nextNode)).length() != 0)
              {
                  m_currentNode = _nextNode;
@@ -1873,7 +1948,7 @@ void KafkaWidget::postprocessCursorPosition()
                 (m_currentNode.nodeName().string().lower() != "br" ||
                  (m_currentNode.nodeName().string().lower() == "br" && /**!m_currentNode.nextSibling().isNull() &&
                             m_currentNode.nextSibling().nodeType() == DOM::Node::TEXT_NODE &&
-                            m_currentNode.nextSibling().nodeValue().string() == "" &&
+                            m_currentNode.nextSibling().nodeValue().string().isEmpty() &&
                             m_currentNode.nextSibling().nextSibling().isNull() &&*/
                   !m_currentNode.previousSibling().isNull() &&
                   m_currentNode.previousSibling().nodeType() == DOM::Node::TEXT_NODE &&
@@ -1886,7 +1961,7 @@ void KafkaWidget::postprocessCursorPosition()
                 if(_nextNode == 0)
                     break;
                 attrs2 = w->getAttrs(_nextNode);
-                if(attrs2->chCurFoc() == kNodeAttrs::textNode &&
+                if(attrs2 && attrs2->chCurFoc() == kNodeAttrs::textNode &&
                         (static_cast<DOM::CharacterData>(_nextNode)).length()
                         != 0)
                 {
@@ -1902,7 +1977,7 @@ void KafkaWidget::postprocessCursorPosition()
 
                     break;
                 }
-                else if(attrs2->chCurFoc() == kNodeAttrs::singleNodeAndItself)
+                else if(attrs2 && attrs2->chCurFoc() == kNodeAttrs::singleNodeAndItself)
                 {
                     m_currentNode = _nextNode;
                     d->m_cursorOffset = 1;
@@ -1929,9 +2004,9 @@ void KafkaWidget::postprocessCursorPosition()
                 if(_nextNode == 0)
                     break;
                 attrs2 = w->getAttrs(_nextNode);
-                if(attrs2->chCurFoc() == kNodeAttrs::singleNodeAndItself)
+                if(attrs2 && attrs2->chCurFoc() == kNodeAttrs::singleNodeAndItself)
                     break;
-                else if(attrs2->chCurFoc() == kNodeAttrs::textNode &&
+                else if(attrs2 && attrs2->chCurFoc() == kNodeAttrs::textNode &&
                         (static_cast<DOM::CharacterData>(_nextNode)).length() != 0)
                 {
                     m_currentNode = _nextNode;
@@ -2000,7 +2075,7 @@ void KafkaWidget::khtmlDrawContentsEvent(khtml::DrawContentsEvent *event)
     KHTMLPart::khtmlDrawContentsEvent(event);
 }
 
-void KafkaWidget::getCurrentNode(DOM::Node &_currentNode, int &offset)
+void KafkaWidget::getCurrentNode(DOM::Node &_currentNode, long &offset)
 {
     _currentNode = m_currentNode;
     offset = d->m_cursorOffset;
@@ -2016,6 +2091,18 @@ void KafkaWidget::setCurrentNode(DOM::Node node, int offset)
     //setCaretPosition(m_currentNode, (long)d->m_cursorOffset);
 }
 
+void KafkaWidget::setCurrentNode(Node* cursorNode, int cursorOffset)
+{
+    DOM::Node domNode;
+    long longDomNodeOffset;
+    KafkaDocument::ref()->translateNodeIntoKafkaCursorPosition(cursorNode, cursorOffset, domNode, longDomNodeOffset);
+    if (!domNode.isNull() && domNode.nodeType() != DOM::Node::TEXT_NODE &&
+         !domNode.firstChild().isNull() && domNode.firstChild().nodeType() == DOM::Node::TEXT_NODE)
+        domNode = domNode.firstChild();
+    if (!domNode.isNull())
+        setCurrentNode(domNode, (int)longDomNodeOffset);       
+}
+
 void KafkaWidget::putCursorAtFirstAvailableLocation()
 {
     kNodeAttrs *attrs = 0L;
@@ -2076,6 +2163,8 @@ void KafkaWidget::slotNewCursorPos(const DOM::Node &domNode, long offset)
 
     if(quantaApp->aTab && ViewManager::ref()->activeView()->hadLastFocus() == QuantaView::VPLFocus)
         quantaApp->aTab->setCurrentNode(w->getNode(domNode));
+
+    updateToggableTagActions(/*domNode, offset*/);            
 }
 
 void KafkaWidget::moveDomNodes(DOM::Node newParent, DOM::Node startNode, DOM::Node endNode,
@@ -2136,7 +2225,7 @@ void KafkaWidget::removeSelection()
     NodeSelectionInd selection;
     selection.fillWithVPLCursorSelection();
     Node* cursorNode = kafkaCommon::getNodeFromLocation(selection.cursorNode());
-    int cursorOffset = 0;
+    long cursorOffset = 0;
     long domNodeCursorOffset = 0;
         
     kafkaCommon::DTDRemoveSelection(selection, &cursorNode, cursorOffset, m_modifs);
@@ -2160,4 +2249,23 @@ void KafkaWidget::removeSelection()
     makeCursorVisible();
 }
 
+void KafkaWidget::applyQueuedToggableTagActions()
+{
+    QStringList queued_actions = quantaApp->tagActionPool();
+    QPtrList<TagAction> action_list = quantaApp->tagActions();
+    for(QStringList::Iterator it = queued_actions.begin(); it != queued_actions.end(); ++it) 
+    {
+        TagAction* tag_action = 0;    
+        for (tag_action = action_list.first(); tag_action; tag_action = action_list.next())
+        {
+            if(tag_action->name() == *it)
+            {
+                tag_action->slotActionActivated(KAction::EmulatedActivation, Qt::NoButton);
+                break;
+            }
+        }
+    }
+    quantaApp->removeAllTagActionPoolItems();
+}
+
 #include "kafkahtmlpart.moc"
