@@ -35,6 +35,7 @@
 #include <qtextstream.h>
 #include <qdatetime.h>
 #include <qclipboard.h> 
+#include <qptrvector.h> 
 
 #include "document.h"
 #include "viewmanager.h"
@@ -347,6 +348,8 @@ kNodeAttrs* KafkaDocument::connectDomNodeToQuantaNode(DOM::Node domNode, Node *n
 		_domNode.nodeName().string());*/
 	else
 		name = domNode.nodeName().string().lower();
+ 
+//     kdDebug(25001) << "KafkaDocument::connectDomNodeToQuantaNode() - domNode name: |" << name << "|" << endl;
 
 	props = new kNodeAttrs();
 
@@ -460,18 +463,18 @@ void KafkaDocument::setCursorAndSelection(NodeSelectionInd *nodeSelection)
   bool cursorAtSelectionStart;
   DOM::Node startDomNode, endDomNode, foo;
   Node *startNode, *endNode;
-  int startOffset, endOffset;
-   
+  long startOffset, endOffset;
+
   cursorAtSelectionStart = nodeSelection->cursorAtSelectionStart();
   startNode = kafkaCommon::getNodeFromLocation(nodeSelection->cursorNode());
   endNode = kafkaCommon::getNodeFromLocation(nodeSelection->cursorNodeEndSel());
-  
+
   if(startNode && startNode->rootNode())
     startDomNode = *(startNode->rootNode());
   if(endNode && endNode->rootNode())
     endDomNode = *(endNode->rootNode());
-  translateNodeIntoKafkaCursorPosition(startNode, nodeSelection->cursorOffset(), foo, (long&)startOffset);
-  translateNodeIntoKafkaCursorPosition(endNode, nodeSelection->cursorOffsetEndSel(), foo, (long&)endOffset);
+  translateNodeIntoKafkaCursorPosition(startNode, nodeSelection->cursorOffset(), foo, startOffset);
+  translateNodeIntoKafkaCursorPosition(endNode, nodeSelection->cursorOffsetEndSel(), foo, endOffset);
   
   
   if(cursorAtSelectionStart && !startDomNode.isNull())
@@ -487,6 +490,18 @@ void KafkaDocument::setCursorAndSelection(NodeSelectionInd *nodeSelection)
     m_kafkaPart->setSelection(DOM::Range(startDomNode, (long)startOffset, endDomNode, (long)endOffset));
 }
 
+void KafkaDocument::setCursor(Node* cursorNode, int cursorOffset)
+{
+    DOM::Node domNode;
+    long longDomNodeOffset;
+    translateNodeIntoKafkaCursorPosition(cursorNode, cursorOffset, domNode, longDomNodeOffset);
+    if (!domNode.isNull() && domNode.nodeType() != DOM::Node::TEXT_NODE &&
+         !domNode.firstChild().isNull() && domNode.firstChild().nodeType() == DOM::Node::TEXT_NODE)
+        domNode = domNode.firstChild();
+    if (!domNode.isNull())
+        m_kafkaPart->setCurrentNode(domNode, (int)longDomNodeOffset);       
+}
+
 bool KafkaDocument::buildKafkaNodeFromNode(Node *node, bool insertNode)
 {
 #ifdef LIGHT_DEBUG
@@ -722,6 +737,7 @@ bool KafkaDocument::buildKafkaNodeFromNode(Node *node, bool insertNode)
 
             if(qTag->isChild(node, false))
             {
+                
                 if(nextNode.isNull())
                 {
                     if(!kafkaCommon::insertDomNode(newNode, parentNode))
@@ -785,10 +801,13 @@ void KafkaDocument::buildNodeFromKafkaNode(Node *node, DOM::Node domNode)
 	if(!node) return;
 
 	node->tag->setCleanStrBuilt(false);
-        node->tag->setIndentationDone(false);
+  node->tag->setIndentationDone(false);
 	if(domNode.nodeType() == DOM::Node::TEXT_NODE)
 	{
-		node->tag->setStr(domNode.nodeValue().string());
+    QString text = domNode.nodeValue().string();
+    text.replace("<", "&lt;");
+    text.replace(">", "&gt;");
+		node->tag->setStr(text);
 	}
 	else
 	{
@@ -866,7 +885,9 @@ QString KafkaDocument::getDecodedChar(const QString &encodedChar)
 QString KafkaDocument::getDecodedText(const QString &a_encodedText, bool translateWhiteSpacesAndLineBreaks,
 		bool removeLeftWhitespaces, bool removeRightWhitespaces)
 {
-        QString encodedText = a_encodedText;
+  QString encodedText = a_encodedText;
+  if (encodedText.isEmpty())
+    encodedText = " ";
 	QString decodedChar;
 	int i, j;
 #ifdef LIGHT_DEBUG
@@ -1060,7 +1081,12 @@ QString KafkaDocument::generateCodeFromNode(Node *node, int bLine, int bCol, int
                     text = ">";
 		}
 		else
-			text = "<" + QuantaCommon::tagCase(node->tag->name) + ">";
+    {
+      if (node->tag->tagStr() == "-->")
+        text = "-->";
+      else
+			 text = "<" + QuantaCommon::tagCase(node->tag->name) + ">";
+    }
 		bCol += text.length();
 		eCol = bCol - 1;
 		eLine = bLine;
@@ -1110,7 +1136,7 @@ QString KafkaDocument::generateCodeFromNode(Node *node, int bLine, int bCol, int
 	return text;
 }
 
-void KafkaDocument::translateQuantaIntoKafkaCursorPosition(uint curLine, uint curCol, DOM::Node &domNode, int &offset)
+void KafkaDocument::translateQuantaIntoKafkaCursorPosition(uint curLine, uint curCol, DOM::Node &domNode, long &offset)
 {
 	Node *node;
 	int bCol, bLine, eCol, eLine, col, line;
@@ -1217,7 +1243,7 @@ void KafkaDocument::translateQuantaIntoKafkaCursorPosition(uint curLine, uint cu
 #endif
 }
 
-void KafkaDocument::translateQuantaIntoNodeCursorPosition(uint line, uint col, Node **node, int &offset)
+void KafkaDocument::translateQuantaIntoNodeCursorPosition(uint line, uint col, Node **node, long &offset)
 {
   int curCol, curLine, beginCol, beginLine;
   QString currentLine;
@@ -1258,7 +1284,7 @@ void KafkaDocument::translateQuantaIntoNodeCursorPosition(uint line, uint col, N
 #endif
 }
 
-void KafkaDocument::translateKafkaIntoNodeCursorPosition(DOM::Node domNode, long domNodeOffset, Node **node, int &offset)
+void KafkaDocument::translateKafkaIntoNodeCursorPosition(DOM::Node domNode, long domNodeOffset, Node **node, long &offset)
 {
 	QString decodedText, encodedChar, encodedText, currentChar;
 	QChar curChar, oldChar;
@@ -1288,6 +1314,15 @@ void KafkaDocument::translateKafkaIntoNodeCursorPosition(DOM::Node domNode, long
 #endif
 		return ;
 	}
+  if(!(*node)->tag)
+  {
+#ifdef HEAVY_DEBUG
+    kdDebug(25001)<< "KafkaDocument::translateKafkaIntoNodeCursorPosition()" <<
+      " - Node tag not found!" << endl;
+#endif
+    (*node) = 0L;
+    return ;
+  }
 
 	//If this node is selected (-1 means selected) then return "node selected" ;-)
 	if(domNodeOffset == -1)
@@ -1931,7 +1966,8 @@ void KafkaDocument::slotDomNodeModified(DOM::Node domNode, NodeModifsSet* modifs
 		modif->setLocation(kafkaCommon::getLocation(node));
 
 		buildNodeFromKafkaNode(node, domNode);
-
+    if (!modifs)
+      modifs = new NodeModifsSet();
 		modifs->addNodeModif(modif);
 	}
 	else
@@ -2258,7 +2294,7 @@ void KafkaDocument::slotCut()
     Node* endNode = kafkaCommon::getNodeFromLocation(selection_ind.cursorNodeEndSel());
     
     DOM::Node cursorDomNode;
-    int cursorOffset;
+    long cursorOffset;
     
     m_kafkaPart->getCurrentNode(cursorDomNode, cursorOffset);
     Node* cursorNode = getNode(cursorDomNode);
@@ -2267,7 +2303,7 @@ void KafkaDocument::slotCut()
 }
 
 void KafkaDocument::slotCut(Node* startNode, int startOffset, Node* endNode, int endOffset, 
-                            Node** cursorNode, int cursorOffset, QString const& text)
+                            Node** cursorNode, long cursorOffset, QString const& text)
 {
     if(!startNode || !endNode)
         return;
@@ -2279,6 +2315,9 @@ void KafkaDocument::slotCut(Node* startNode, int startOffset, Node* endNode, int
     
     m_currentDoc->docUndoRedo->addNewModifsSet(modifs, undoRedo::NodeTreeModif);
     
+    //Now update the VPL cursor position
+    kafkaWidget->setCurrentNode(startNode, startOffset);
+    
     if(subtree_root)
     {        
         KafkaDragObject* node_drag = new KafkaDragObject(subtree_root);
@@ -2348,7 +2387,7 @@ void KafkaDocument::slotPaste()
         selection_ind.fillWithVPLCursorSelection();
     
         Node* cursorNode = kafkaCommon::getNodeFromLocation(selection_ind.cursorNode());
-        int cursorOffset = selection_ind.cursorOffset();
+        long cursorOffset = selection_ind.cursorOffset();
         
         NodeModifsSet *modifs = new NodeModifsSet();
         
@@ -2359,5 +2398,8 @@ void KafkaDocument::slotPaste()
         kafkaCommon::DTDInsertNodeSubtree(node, selection_ind, &cursorNode, cursorOffset, modifs);
         
         m_currentDoc->docUndoRedo->addNewModifsSet(modifs, undoRedo::NodeTreeModif, 0, false);
+        
+        //Now update the VPL cursor position
+        kafkaWidget->setCurrentNode(cursorNode, cursorOffset);
     }
 }
