@@ -93,6 +93,17 @@ QStringList Project::fileNameList()
   return list;
 }
 
+KURL::List Project::files()
+{
+  KURL::List list;
+  ProjectList::Iterator  it( d->m_projectFiles );
+  for ( ; it.current(); ++it) {
+    if (!it.current()->fileName(false).isEmpty())
+      list.append((*it.current()));
+  }
+  return list;
+}
+
 void Project::insertFile(const KURL& nameURL, bool repaint )
 {
   if (d->excludeRx.exactMatch(nameURL.path()))
@@ -160,6 +171,7 @@ void Project::loadLastProject(bool reload)
   d->config->setGroup("Projects");
   QStringList projectList = d->config->readPathListEntry("OpenProjects");
   QStringList tempList = d->config->readPathListEntry("ProjectTempFiles");
+  QStringList sessionTempList = d->config->readPathListEntry("ProjectSessionTempFiles");
   // remove all local open projects because project and temp file are the same
   for (uint i = 0; i < projectList.count(); ++i)
   {
@@ -169,6 +181,9 @@ void Project::loadLastProject(bool reload)
       QStringList::Iterator it = tempList.at(i);
       if (it != tempList.end())
         tempList.remove(it);
+      QStringList::Iterator it2 = sessionTempList.at(i);
+      if (it2 != sessionTempList.end())
+        sessionTempList.remove(it2);
       --i;
     }
   }
@@ -180,25 +195,29 @@ void Project::loadLastProject(bool reload)
     urlPath = projectList[0];
     QuantaCommon::setUrl(url, urlPath);
     QString tempPath = tempList[0];
+    QString sessionTempPath = sessionTempList[0];
     // test if the remote project is available
     if ( KIO::NetAccess::exists(url, false, d->m_mainWindow) )
     {
       KURL tempURL = KURL().fromPathOrURL(tempPath);
       if (KIO::NetAccess::exists(tempURL, false, d->m_mainWindow) &&
-          KMessageBox::questionYesNo(d->m_mainWindow, i18n("<qt>Found a backup for project <b>%1</b>.<br> Do you want to open it?</qt>").arg(url.prettyURL()), i18n("Open Project Backup") )
+          KMessageBox::questionYesNo(d->m_mainWindow, i18n("<qt>Found a backup for project <b>%1</b>.<br> Do you want to open it?</qt>").arg(url.prettyURL()), i18n("Open Project Backup"), KStdGuiItem::open(), i18n("Do Not Open") )
           == KMessageBox::Yes)
       {
         d->m_tmpProjectFile = tempPath;
-        d->loadProjectFromTemp(url, d->m_tmpProjectFile);
+        d->m_tmpSessionFile = sessionTempPath;
+        d->loadProjectFromTemp(url, d->m_tmpProjectFile, d->m_tmpSessionFile);
         // the lists might have changed
         d->config->writePathEntry("OpenProjects", projectList);
         d->config->writePathEntry("ProjectTempFiles", tempList);
+        d->config->writePathEntry("ProjectSessionTempFiles", sessionTempList);
         d->config->sync();
         return;
       }
     }
-    projectList.remove( projectList.at(0) );
-    tempList.remove( tempList.at(0) );
+    projectList.remove(projectList.at(0));
+    tempList.remove(tempList.at(0));
+    sessionTempList.remove(sessionTempList.at(0));
   }
   // the lists might have changed
   d->config->writePathEntry("OpenProjects", projectList);
@@ -221,13 +240,14 @@ void Project::loadLastProject(bool reload)
 
 void Project::slotOpenProject(const KURL &url)
 {
+  kdDebug(24000) << "Open recent project: " << url <<endl;
   if ( !url.isEmpty() )
   {
-    if ( !QExtFileInfo::exists(url) )
+    if ( !QExtFileInfo::exists(url, d->m_mainWindow) )
     {
       emit hideSplash();
       if (KMessageBox::questionYesNo(d->m_mainWindow,
-           i18n("<qt>The file <b>%1</b> does not exist.<br> Do you want to remove it from the list?</qt>").arg(url.prettyURL(0, KURL::StripFileProtocol)) )
+           i18n("<qt>The file <b>%1</b> does not exist.<br> Do you want to remove it from the list?</qt>").arg(url.prettyURL(0, KURL::StripFileProtocol)), QString::null, KStdGuiItem::del(), i18n("Keep") )
            == KMessageBox::Yes)
       {
         d->m_projectRecent->removeURL(url);
@@ -386,16 +406,15 @@ void Project::slotRemove(const KURL& urlToRemove)
 
   if (d->m_modified)
     setModified();  // there happens more than setting the flag !
-  emit closeFile(urlToRemove);
   emit reloadTree( &(d->m_projectFiles), false, QStringList() );
   emit newStatus();
   
   QString urlPath = QExtFileInfo::toRelative(urlToRemove, d->baseURL).path();
   QString nice = urlPath;
   nice = KStringHandler::lsqueeze(nice, 60);
-  if (KMessageBox::warningYesNo(d->m_mainWindow, i18n("<qt>Do you want to remove <br><b>%1</b><br> from the server(s) as well?</qt>").arg(nice), i18n("Remove From Server"), KStdGuiItem::yes(), KStdGuiItem::no(), "RemoveFromServer") == KMessageBox::Yes )
+  if (KMessageBox::warningContinueCancel(d->m_mainWindow, i18n("<qt>Do you want to remove <br><b>%1</b><br> from the server(s) as well?</qt>").arg(nice), i18n("Remove From Server"), KStdGuiItem::remove(), "RemoveFromServer") == KMessageBox::Continue )
   {
-    QDomNode profilesNode = d->dom.firstChild().firstChild().namedItem("uploadprofiles");
+    QDomNode profilesNode = d->m_sessionDom.firstChild().firstChild().namedItem("uploadprofiles");  
     QDomNodeList profileList = profilesNode.toElement().elementsByTagName("profile");
     QDomElement e;
     QString s;
@@ -468,6 +487,8 @@ void Project::slotOptions()
       optionsPage.buttonDebuggerOptions->setEnabled(true);
     }
   }
+  optionsPage.checkDebuggerPersistentBreakpoints->setChecked(d->m_debuggerPersistentBreakpoints);
+  optionsPage.checkDebuggerPersistentWatches->setChecked(d->m_debuggerPersistentWatches);
 
   QString excludeStr;
   for (uint i = 0; i < d->excludeList.count(); i++)
@@ -532,7 +553,7 @@ void Project::slotOptions()
   UploadProfilesPage uploadProfilesPage(page);
   topLayout = new QVBoxLayout( page, 0, KDialog::spacingHint() );
   topLayout->addWidget(&uploadProfilesPage);
-  QDomElement uploadEl = d->dom.firstChild().firstChild().namedItem("uploadprofiles").toElement();
+  QDomElement uploadEl = d->m_sessionDom.firstChild().firstChild().namedItem("uploadprofiles").toElement();
   uploadProfilesPage.profileLabel->setText(uploadEl.attribute("defaultProfile"));
   uploadProfilesPage.checkShowUploadTreeviews->setChecked(d->m_showUploadTreeviews);
 
@@ -546,25 +567,26 @@ void Project::slotOptions()
   if (!teamLeader().name.isEmpty())
   {
     TeamMember member = teamLeader();
-    item = new QListViewItem(membersPage.membersListView, member.name, member.email, i18n("Team Leader"), member.task);
+    item = new QListViewItem(membersPage.membersListView, member.name, member.nickName, member.email, i18n("Team Leader"), member.task);
     membersPage.membersListView->insertItem(item);
   }
   for (QMap<QString, TeamMember>::ConstIterator it = d->m_subprojectLeaders.constBegin(); it != d->m_subprojectLeaders.constEnd(); ++it)
   {
     TeamMember member = it.data();
-    item = new QListViewItem(membersPage.membersListView, member.name, member.email, i18n("Subproject Leader"), member.task, it.key());
+    item = new QListViewItem(membersPage.membersListView, member.name, member.nickName, member.email, i18n("Subproject Leader"), member.task, it.key());
   }
   for (QMap<QString, TeamMember>::ConstIterator it = d->m_taskLeaders.constBegin(); it != d->m_taskLeaders.constEnd(); ++it)
   {
     TeamMember member = it.data();
-    item = new QListViewItem(membersPage.membersListView, member.name, member.email, i18n("Task Leader"), it.key());
+    item = new QListViewItem(membersPage.membersListView, member.name, member.nickName, member.email, i18n("Task Leader"), it.key());
   }
   for (QValueList<TeamMember>::ConstIterator it = d->m_simpleMembers.constBegin(); it != d->m_simpleMembers.constEnd(); ++it)
   {
     TeamMember member = *it;
-    item = new QListViewItem(membersPage.membersListView, member.name, member.email, i18n("Simple Member"), member.task);
+    item = new QListViewItem(membersPage.membersListView, member.name, member.nickName, member.email, i18n("Simple Member"), member.task);
   }
   membersPage.mailingListEdit->setText(d->m_mailingList);
+  membersPage.setYourself(d->m_yourself);
 
 //add the event configuration page
   page = optionsDlg.addPage(i18n("Event Configuration"));
@@ -582,6 +604,8 @@ void Project::slotOptions()
 
     // Debugger
     d->debuggerClient = optionsPage.comboDebuggerClient->currentText();
+    d->m_debuggerPersistentBreakpoints = optionsPage.checkDebuggerPersistentBreakpoints->isChecked();
+    d->m_debuggerPersistentWatches = optionsPage.checkDebuggerPersistentWatches->isChecked();
 
     d->m_defaultDTD = DTDs::ref()->getDTDNameFromNickName(optionsPage.dtdCombo->currentText()).lower();
     d->m_defaultEncoding  = optionsPage.encodingCombo->currentText();
@@ -589,7 +613,7 @@ void Project::slotOptions()
     QuantaCommon::setUrl(d->templateURL, optionsPage.linePrjTmpl->text());
     d->templateURL.adjustPath(1);
     d->templateURL = QExtFileInfo::toAbsolute(d->templateURL, d->baseURL);
-    if (!QExtFileInfo::createDir(d->templateURL))
+    if (!QExtFileInfo::createDir(d->templateURL, d->m_mainWindow))
     {
       QuantaCommon::dirCreationError(d->m_mainWindow, d->templateURL);
     }
@@ -597,7 +621,7 @@ void Project::slotOptions()
     QuantaCommon::setUrl(d->toolbarURL, optionsPage.linePrjToolbar->text());
     d->toolbarURL.adjustPath(1);
     d->toolbarURL = QExtFileInfo::toAbsolute(d->toolbarURL, d->baseURL);
-    if (!QExtFileInfo::createDir(d->toolbarURL))
+    if (!QExtFileInfo::createDir(d->toolbarURL, d->m_mainWindow))
     {
       QuantaCommon::dirCreationError(d->m_mainWindow, d->toolbarURL);
     }
@@ -611,10 +635,11 @@ void Project::slotOptions()
 
     el = projectNode.toElement();
     el.setAttribute("name",d->projectName);
+    el.setAttribute("encoding", d->m_defaultEncoding);
+    el = d->m_sessionDom.firstChild().firstChild().toElement();
     el.setAttribute("previewPrefix", d->previewPrefix.url() );
     el.setAttribute("usePreviewPrefix", d->usePreviewPrefix );
     el.setAttribute("usePersistentBookmarks", d->m_persistentBookmarks);
-    el.setAttribute("encoding", d->m_defaultEncoding);
 
     el = projectNode.namedItem("author").toElement();
     if (!el.isNull())
@@ -637,6 +662,8 @@ void Project::slotOptions()
     el =d->dom.createElement("debuggerclient");
     projectNode.appendChild( el );
     el.appendChild(d->dom.createTextNode( d->debuggerClient ) );
+    el.setAttribute("persistentBreakpoints", d->m_debuggerPersistentBreakpoints);
+    el.setAttribute("persistentWatches", d->m_debuggerPersistentWatches);
 
     d->m_excludeCvsignore = optionsPage.checkCvsignore->isChecked();
     excludeStr = optionsPage.lineExclude->text();
@@ -719,7 +746,7 @@ void Project::slotOptions()
     while (it.current())
     {
         item = it.current();
-        QString role = item->text(2);
+        QString role = item->text(3);
         if (role == i18n(teamLeaderStr.utf8()))
         {
            QDomElement leaderEl = d->dom.createElement("leader");
@@ -727,13 +754,16 @@ void Project::slotOptions()
            el = d->dom.createElement("name");
            leaderEl.appendChild(el);
            el.appendChild(d->dom.createTextNode(item->text(0)));
-           el = d->dom.createElement("email");
+           el = d->dom.createElement("nickName");
            leaderEl.appendChild(el);
            el.appendChild(d->dom.createTextNode(item->text(1)));
+           el = d->dom.createElement("email");
+           leaderEl.appendChild(el);
+           el.appendChild(d->dom.createTextNode(item->text(2)));
         } else
         if (role == i18n(subprojectLeaderStr.utf8()))
         {
-           QString prjName = item->text(4);
+           QString prjName = item->text(5);
            savedSubprojects.append(prjName);
            QDomElement subEl = d->dom.createElement("subproject");
            for (uint i = 0; i < d->m_subprojects.count(); i++)
@@ -748,28 +778,33 @@ void Project::slotOptions()
            subLeadersNode.appendChild(subEl);
            el = d->dom.createElement("subprojectleader");
            el.setAttribute("name", item->text(0));
-           el.setAttribute("email", item->text(1));
+           el.setAttribute("nickName", item->text(1));
+           el.setAttribute("email", item->text(2));
            subEl.appendChild(el);
         } else
         if (role == i18n(taskLeaderStr.utf8()))
         {
            el = d->dom.createElement("projecttask");
            el.setAttribute("tasklead", item->text(0));
-           el.setAttribute("email", item->text(1));
-           el.setAttribute("task", item->text(3));
+           el.setAttribute("nickName", item->text(1));
+           el.setAttribute("email", item->text(2));
+           el.setAttribute("task", item->text(4));
            taskLeadersNode.appendChild(el);
         } else
         if (role == i18n(simpleMemberStr.utf8()))
         {
            QDomElement memberEl = d->dom.createElement("member");
-           memberEl.setAttribute("task", item->text(3));
+           memberEl.setAttribute("task", item->text(4));
            teamNode.appendChild(memberEl);
            el = d->dom.createElement("name");
            memberEl.appendChild(el);
            el.appendChild(d->dom.createTextNode(item->text(0)));
-           el = d->dom.createElement("email");
+           el = d->dom.createElement("nickName");
            memberEl.appendChild(el);
            el.appendChild(d->dom.createTextNode(item->text(1)));
+           el = d->dom.createElement("email");
+           memberEl.appendChild(el);
+           el.appendChild(d->dom.createTextNode(item->text(2)));
         }
         ++it;
     }
@@ -788,6 +823,13 @@ void Project::slotOptions()
     el.setAttribute("address", membersPage.mailingListEdit->text());
     teamNode.appendChild(el);
     projectNode.appendChild(teamNode);
+    teamNode = d->m_sessionDom.firstChild().namedItem("teamdata");
+    if (!teamNode.isNull())
+      d->m_sessionDom.firstChild().removeChild(teamNode);
+    d->m_yourself = membersPage.yourself();
+    el = d->m_sessionDom.createElement("teamdata");
+    el.setAttribute("yourself", d->m_yourself);
+    d->m_sessionDom.firstChild().appendChild(el);
 
     eventsPage.saveEvents(d->dom);
     d->m_eventsEnabled = eventsPage.enableEventsBox->isChecked();
@@ -1048,11 +1090,25 @@ QDomDocument* Project::dom()
   return &d->dom;
 }
 
+QDomDocument* Project::sessionDom()
+{
+  return &d->m_sessionDom;
+}
+
 QString Project::debuggerClient()
 {
   return d->debuggerClient;
 }
 
+bool Project::debuggerPersistentBreakpoints( )
+{
+  return d->m_debuggerPersistentBreakpoints;
+}
+
+bool Project::debuggerPersistentWatches( )
+{
+  return d->m_debuggerPersistentWatches;
+}
 
 void Project::setModified(bool b)
 {
@@ -1069,6 +1125,35 @@ EventActions* Project::events()
     return 0L;
 }
 
+QString Project::yourself()
+{
+  return d->m_yourself;
+}
+
+QStringList Project::yourRoles()
+{
+  QStringList roles;
+  if (d->m_yourself.isEmpty())
+    return roles;
+  QString yourNick = d->m_yourself.lower();
+  if (d->m_teamLeader.nickName.lower() == yourNick)
+    roles += "team leader";
+  QValueList<SubProject>::ConstIterator end =  d->m_subprojects.constEnd();
+  for (QValueList<SubProject>::ConstIterator it = d->m_subprojects.constBegin(); it != end; ++it)
+  {
+    if (subprojectLeader((*it).name).nickName.lower() == yourNick)
+      roles += "subproject leader:" + (*it).name.lower();
+  }
+
+   for (QMap<QString, TeamMember>::ConstIterator it = d->m_taskLeaders.constBegin(); it != d->m_taskLeaders.constEnd(); ++it)
+   {
+      if (it.data().nickName.lower() == yourNick)
+        roles += "task leader:" + it.key().lower();
+   }
+
+  return roles;
+}
+
 TeamMember Project::teamLeader()
 {
   return d->m_teamLeader;
@@ -1152,7 +1237,7 @@ bool Project::queryClose()
     canExit = d->uploadProjectFile();
     if (! canExit)
     {
-      if (KMessageBox::warningYesNo(d->m_mainWindow, i18n("Saving of project failed. Do you want to continue with exit (might cause data loss)?"), i18n("Project Saving Error")) == KMessageBox::Yes)
+      if (KMessageBox::warningContinueCancel(d->m_mainWindow, i18n("Saving of project failed. Do you want to continue with exit (might cause data loss)?"), i18n("Project Saving Error"),KStdGuiItem::quit()) == KMessageBox::Continue)
           canExit = true;
     }
     if (canExit)
@@ -1235,4 +1320,5 @@ void Project::saveBookmarks(const KURL &url, KTextEditor::MarkInterface *markIf)
   }
 }
 
+
 #include "project.moc"
