@@ -448,8 +448,23 @@ void Document::insertText(const QString &a_text, bool adjustCursor, bool reparse
       if (s[i] == '"' && (i == 0 || s[i-1] != '\\'))
         insideQuotes = !insideQuotes;
     }
-    if (insideQuotes)
+    int eLine, eCol;
+    n->tag->endPos(eLine, eCol);
+    s = this->text(line + 1, col, eLine, eCol);
+    bool closeQuotationFound = false;
+    for (int i = 0 ; i < (int)s.length() - 1; i++)
+    {
+      if (s[i] == '"' && (i == 0 || s[i-1] != '\\'))
+      {
+        closeQuotationFound = true;
+        break;
+      }
+    }
+    if (insideQuotes && closeQuotationFound)
+    {
+      text.replace("\\\"", "\"");
       text.replace("\"", "\\\"");
+    }
   }
 
   editIf->insertText(line, col, text);
@@ -1083,8 +1098,8 @@ QValueList<KTextEditor::CompletionEntry>* Document::getGroupCompletions(Node *no
       {
         if (elementList[i]->parentNode == 0L || elementList[i]->global)
         {
-          completion.text = it.key().section('|', -1);
-          completions->append( completion );
+          completion.text = it.key().section('|', -1).stripWhiteSpace();
+          completions->append(completion);
           break;
         } else
         {
@@ -1095,8 +1110,8 @@ QValueList<KTextEditor::CompletionEntry>* Document::getGroupCompletions(Node *no
           }
           if (n == elementList[i]->parentNode)
           {
-            completion.text = it.key().section('|', -1);
-            completions->append( completion );
+            completion.text = it.key().section('|', -1).stripWhiteSpace();
+            completions->append(completion);
             break;
           }
         }
@@ -1113,8 +1128,8 @@ QValueList<KTextEditor::CompletionEntry>* Document::getGroupCompletions(Node *no
     {
       if (list[i].startsWith(word) && list[i] != word)
       {
-        completion.text = list[i];
-        completions->append( completion );
+        completion.text = list[i].stripWhiteSpace();
+        completions->append(completion);
       }
     }
   }
@@ -1285,7 +1300,7 @@ QValueList<KTextEditor::CompletionEntry>* Document::getTagCompletions(int line,
       completion.text = QuantaCommon::tagCase(tagNameList[i]);
     else
       completion.text = tagNameList[i];
-    completion.text = completion.text.left(completion.text.length() - 10);
+    completion.text = completion.text.left(completion.text.length() - 10).stripWhiteSpace();
     completion.comment = comments[tagNameList[i]];
     completions->append( completion );
   }
@@ -2196,6 +2211,29 @@ void Document::save()
 //  kdDebug(24000) << "Document " << url() << " saved." << endl;
 }
 
+bool Document::saveAs(const KURL& url)
+{
+  bool result = m_doc->saveAs(url);
+  if (result)
+  {
+    m_md5sum = "";
+    if (url.isLocalFile())
+    {
+      QFile f(url.path());
+      if (f.open(IO_ReadOnly))
+      {
+        const char* c = "";
+        KMD5 context(c);
+        context.reset();
+        context.update(f);
+        m_md5sum = context.hexDigest();
+        f.close();
+      }
+    }
+  }
+  return result;
+}
+
 void Document::enableGroupsForDTEP(const QString& dtepName, bool enable)
 {
   if (m_groupsForDTEPs.isEmpty())
