@@ -57,6 +57,7 @@
 #include <kio/netaccess.h>
 #include <ktabwidget.h>
 #include <kmultitabbar.h>
+#include <ktexteditor/view.h>
 
 #include "wkafkapart.h"
 #include "kafkacommon.h"
@@ -78,6 +79,7 @@
 
 #include "wkafkapart.h"
 #include "whtmlpart.h"
+#include "annotationoutput.h"
 #include "messageoutput.h"
 
 #include "tagaction.h"
@@ -170,6 +172,8 @@ void QuantaInit::initQuanta()
   m_quanta->m_debugger = new DebuggerManager(m_quanta);
   connect(Project::ref(), SIGNAL(newProjectLoaded(const QString &, const KURL &, const KURL &)),
           m_quanta->m_debugger, SLOT(slotNewProjectLoaded(const QString &, const KURL &, const KURL &)));
+  connect(Project::ref(), SIGNAL(eventHappened(const QString &, const QString &, const QString &)),
+          m_quanta->m_debugger, SLOT(slotHandleEvent(const QString &, const QString &, const QString &)));
   connect(m_quanta->m_debugger, SIGNAL(hideSplash()), m_quanta, SLOT(slotHideSplash()));
 
   m_quanta->m_pluginInterface->readConfig(); //call here, so the plugin actions are created before the GUI
@@ -186,6 +190,7 @@ void QuantaInit::initQuanta()
   addToolTreeView(m_quanta->scriptTab, i18n("Scripts"), BarIcon("run"), KDockWidget::DockLeft);
   m_quanta->m_messageOutputView = addToolTreeView(m_quanta->m_messageOutput, i18n("Messages"), SmallIcon("openterm"), KDockWidget::DockBottom);
   m_quanta->m_problemsOutputView = addToolTreeView(m_quanta->m_problemOutput, i18n("Problems"), SmallIcon("info"), KDockWidget::DockBottom);
+  m_quanta->m_annotationOutputView = addToolTreeView(m_quanta->m_annotationOutput, i18n("Annotations"), SmallIcon("stamp"), KDockWidget::DockBottom);
 
   // Restore the dock layout
   m_config->setGroup  ("General Options");
@@ -258,14 +263,10 @@ void QuantaInit::initQuanta()
           m_quanta, SLOT(gotoFileAndLine(const QString&, int, int)));
   connect(m_quanta->m_problemOutput, SIGNAL(clicked(const QString&, int, int)),
           m_quanta, SLOT(gotoFileAndLine(const QString&, int, int)));
-
-  m_quanta->refreshTimer = new QTimer(m_quanta);
-  connect(m_quanta->refreshTimer, SIGNAL(timeout()), m_quanta, SLOT(slotReparse()));
-  m_quanta->refreshTimer->start( qConfig.refreshFrequency*1000, false ); //update the structure tree every 5 seconds
-  if (qConfig.instantUpdate || qConfig.refreshFrequency == 0)
-  {
-    m_quanta->refreshTimer->stop();
-  }
+  connect(m_quanta->m_annotationOutput->currentFileAnnotations(), SIGNAL(clicked(const QString&, int, int)),
+          m_quanta, SLOT(gotoFileAndLine(const QString&, int, int)));
+  connect(m_quanta->m_annotationOutput, SIGNAL(clicked(const QString&, int, int)),
+          m_quanta, SLOT(gotoFileAndLine(const QString&, int, int)));
 
   m_quanta->slotFileNew();
   m_quanta->slotNewStatus();
@@ -303,6 +304,16 @@ void QuantaInit::initQuanta()
   QExtFileInfo::copy(KURL().fromPathOrURL(qConfig.globalDataDir + resourceDir + "scripts/info.css"), KURL().fromPathOrURL(infoCss));
 
   checkRuntimeDependencies();
+  ViewManager::ref()->activeDocument()->view()->setFocus();
+  
+  m_quanta->refreshTimer = new QTimer(m_quanta);
+  connect(m_quanta->refreshTimer, SIGNAL(timeout()), m_quanta, SLOT(slotReparse()));
+  m_quanta->refreshTimer->start( qConfig.refreshFrequency*1000, false ); //update the structure tree every 5 seconds
+  if (qConfig.instantUpdate || qConfig.refreshFrequency == 0)
+  {
+    m_quanta->refreshTimer->stop();
+  }
+
 }
 
 void QuantaInit::initToolBars()
@@ -415,7 +426,7 @@ void QuantaInit::initProject()
   connect(m_project, SIGNAL(newProjectLoaded(const QString &, const KURL &, const KURL &)),
           m_quanta->fTab, SLOT(slotNewProjectLoaded(const QString &, const KURL &, const KURL &)));
   connect(m_project, SIGNAL(newProjectLoaded(const QString &, const KURL &, const KURL &)),
-          m_quanta->dTab, SLOT(slotNewProjectLoaded(const QString &, const KURL &, const KURL &)));
+          m_quanta->annotationOutput(), SLOT(updateAnnotations()));
 
   connect(pTab, SIGNAL(changeFileDescription(const KURL&, const QString&)),
           m_project, SLOT(slotFileDescChanged(const KURL&, const QString&)));
@@ -436,14 +447,16 @@ void QuantaInit::initView()
   connect(m_quanta, SIGNAL(viewActivated (KMdiChildView *)), m_viewManager, SLOT(slotViewActivated(KMdiChildView*)));
   connect(m_quanta, SIGNAL(lastChildViewClosed()), m_viewManager, SLOT(slotLastViewClosed()));
 //   connect(m_quanta, SIGNAL(viewDeactivated(KMdiChildView *)), m_viewManager, SLOT(slotViewDeactivated(KMdiChildView*)));
-   KafkaDocument *m_kafkaDocument = KafkaDocument::ref(0, 0, "KafkaPart");
-   m_kafkaDocument->getKafkaWidget()->view()->setMinimumHeight(50);
-   m_kafkaDocument->readConfig(quantaApp->config());
-   loadVPLConfig();
+  KafkaDocument *m_kafkaDocument = KafkaDocument::ref(0, 0, "KafkaPart");
+  m_kafkaDocument->getKafkaWidget()->view()->setMinimumHeight(50);
+  m_kafkaDocument->readConfig(quantaApp->config());
+  loadVPLConfig();
   ToolbarTabWidget *toolBarTab = ToolbarTabWidget::ref(quantaApp);
+  connect(toolBarTab, SIGNAL(iconTextModeChanged()), quantaApp, SLOT(slotRefreshActiveWindow()));
+   
   //set the toolview and close button style before the GUI is created
   m_config->setGroup("General Options");
-  int iconTextMode = KToolBar::iconTextSetting();
+  int iconTextMode = m_config->readNumEntry("IconTextMode", KToolBar::IconOnly);
   toolBarTab->setIconText(KToolBar::IconText(iconTextMode));
   qConfig.toolviewTabs = m_config->readNumEntry("MDI style", KMdi::IconOnly);
   m_quanta->initTabWidget();
@@ -462,9 +475,13 @@ void QuantaInit::initView()
   m_quanta->m_messageOutput = new MessageOutput(m_quanta, "Messages");
   m_quanta->m_messageOutput->setFocusPolicy(QWidget::NoFocus);
   m_quanta->m_messageOutput->showMessage(i18n("Message Window..."));
+  connect(m_quanta, SIGNAL(showMessage(const QString&, bool)), m_quanta->m_messageOutput, SLOT(showMessage(const QString&, bool)));
+  connect(m_quanta, SIGNAL(clearMessages()), m_quanta->m_messageOutput, SLOT(clear()));
 
   m_quanta->m_problemOutput = new MessageOutput(m_quanta, "Problems");
   m_quanta->m_problemOutput->setFocusPolicy(QWidget::NoFocus);
+  m_quanta->m_annotationOutput = new AnnotationOutput(m_quanta, "Annotations");
+  m_quanta->m_annotationOutput->setFocusPolicy(QWidget::NoFocus);
 
   m_quanta->createPreviewPart();
   m_quanta->createDocPart();
@@ -550,6 +567,8 @@ void QuantaInit::readOptions()
   qConfig.useAutoCompletion = m_config->readBoolEntry("Auto completion",true);
   qConfig.updateClosingTags = m_config->readBoolEntry("Update Closing Tags", true);
   qConfig.replaceAccented = m_config->readBoolEntry("Replace Accented Chars", false);
+  qConfig.replaceNotInEncoding = m_config->readBoolEntry("Replace Chars Not In Current Encoding", true);
+    
 
   qConfig.defaultEncoding = m_config->readEntry("Default encoding", "UTF8");
 
@@ -607,8 +626,11 @@ void QuantaInit::openLastFiles()
   // Reload previously opened files only if setting allows
   m_config->setGroup("General Options");
   if (!m_config->readBoolEntry("Reload Files", true))
+  {
+    m_quanta->setParserEnabled(true);
+    m_quanta->reparse(true);
     return;
-
+  }
   // we need to check config
   // because project now can be
   // in load stage ( remote prj )
@@ -674,6 +696,7 @@ void QuantaInit::loadInitialProject(const QString& url)
 void QuantaInit::initActions()
 {
     KActionCollection *ac = m_quanta->actionCollection();
+    new KAction(i18n("Annotate..."), 0, m_quanta, SLOT(slotAnnotate()),ac, "annotate"); 
     m_quanta->editTagAction = new KAction( i18n( "&Edit Current Tag..." ), CTRL+Key_E,
                         m_quanta, SLOT( slotEditCurrentTag() ),
                         ac, "edit_current_tag" );
@@ -703,7 +726,6 @@ void QuantaInit::initActions()
     //
     KStdAction::openNew( m_quanta, SLOT( slotFileNew()  ), ac);
     KStdAction::open   ( m_quanta, SLOT( slotFileOpen() ), ac, "file_open");
-    KStdAction::close  ( m_quanta, SLOT( slotFileClose()), ac);
     (void) new KAction(i18n("Close Other Tabs"), 0, ViewManager::ref(), SLOT(slotCloseOtherTabs()), ac, "close_other_tabs");
 
     m_quanta->fileRecent =  KStdAction::openRecent(m_quanta, SLOT(slotFileOpenRecent(const KURL&)),
@@ -782,6 +804,9 @@ void QuantaInit::initActions()
     (void) new KAction( i18n( "&Load && Convert DTD..." ), 0,
                         DTDs::ref(), SLOT( slotLoadDTD() ),
                         ac, "load_dtd" );
+    (void) new KAction( i18n( "Load DTD E&ntities..." ), 0,
+                        DTDs::ref(), SLOT( slotLoadEntities() ),
+                        ac, "load_entities" );
 
     (void) new KAction( i18n( "Load DTD &Package (DTEP)..." ), 0,
                         m_quanta, SLOT( slotLoadDTEP() ),
@@ -806,10 +831,10 @@ void QuantaInit::initActions()
                         m_quanta, SLOT( slotDocumentProperties() ),
                         ac, "tools_document_properties" );
 
-/*    (void) new KAction ( i18n ("F&ormat XML Code"), 0,
+    (void) new KAction ( i18n ("F&ormat XML Code"), 0,
                         m_quanta, SLOT( slotCodeFormatting() ),
                         ac, "tools_code_formatting");
-*/
+
     (void) new KAction( i18n( "&Convert Tag && Attribute Case..."), 0,
                         m_quanta, SLOT(slotConvertCase()),
                         ac, "tools_change_case");
@@ -924,7 +949,7 @@ void QuantaInit::initActions()
         while( !n.isNull() ) {
           QDomElement e = n.toElement(); // try to convert the node to an element.
           if( !e.isNull() ) { // the node was really an element.
-              new TagAction( &e, ac);
+              new TagAction(&e, m_quanta);
           }
           n = n.nextSibling();
         }
@@ -950,7 +975,7 @@ void QuantaInit::initActions()
             if( !e.isNull())
             { // the node was really an element.
                 delete ac->action(e.attribute("name"));
-                new TagAction( &e, ac);
+                new TagAction(&e, m_quanta);
             }
             n = n.nextSibling();
           }
@@ -979,6 +1004,11 @@ void QuantaInit::initActions()
                         ac, "view_with_konqueror" );
     act->plug(m_quanta->showPreviewAction->popupMenu());
 
+
+    act = ac->action("view_with_firefox");
+    if (act)
+      act->plug(m_quanta->showPreviewAction->popupMenu());
+
     act = ac->action("view_with_mozilla");
     if (act)
       act->plug(m_quanta->showPreviewAction->popupMenu());
@@ -1232,7 +1262,8 @@ void QuantaInit::recoverCrashed(QStringList& recoveredFileNameList)
 
 QString QuantaInit::searchPathListEntry(const QString& backedUpUrl,const QString& autosavedUrls)
 {
-  KURL k(backedUpUrl);
+  KURL k;
+  QuantaCommon::setUrl(k, backedUpUrl);
   QStringList autosavedUrlsList = QStringList::split(",", autosavedUrls);
   QStringList::Iterator autosavedUrlsIt;
   for (autosavedUrlsIt = autosavedUrlsList.begin();
