@@ -379,7 +379,7 @@ void ProjectTreeView::slotCreateFolder()
        url.setPath(url.path() + "/" + folderName + "/");
      else
        url.setPath(url.directory() + "/" + folderName +"/");
-     if (QExtFileInfo::createDir(url))
+     if (QExtFileInfo::createDir(url, this))
      {
         emit insertToProject(url);
      }
@@ -397,7 +397,7 @@ void ProjectTreeView::slotCreateFile()
       url.setPath(url.path() + "/" + fileName);
     else
       url.setPath(url.directory() + "/" + fileName);
-    if (QExtFileInfo::exists(url))
+    if (QExtFileInfo::exists(url, this))
     {
       KMessageBox::error(this, i18n("<qt>Cannot create file, because a file named <b>%1</b> already exists.</qt>").arg(fileName), i18n("Error Creating File"));
       return;
@@ -405,7 +405,7 @@ void ProjectTreeView::slotCreateFile()
     KTempFile *tempFile = new KTempFile(tmpDir);
     tempFile->setAutoDelete(true);
     tempFile->close();
-    if (QExtFileInfo::copy(KURL::fromPathOrURL(tempFile->name()), url));
+    if (QExtFileInfo::copy(KURL::fromPathOrURL(tempFile->name()), url))
     {
       emit insertToProject(url);
       emit openFile(url);
@@ -424,7 +424,7 @@ void ProjectTreeView::slotRemoveFromProject(int askForRemove)
     QString nice = QExtFileInfo::toRelative(url, m_projectBaseURL).path();
     nice = KStringHandler::lsqueeze(nice, 60);
     if ( !askForRemove ||
-         KMessageBox::warningContinueCancel(this,i18n("<qt>Do you really want to remove <br><b>%1</b><br> from the project?</qt>").arg(nice), i18n("Remove From Project"), KStdGuiItem::del(), "RemoveFromProject") == KMessageBox::Continue )
+         KMessageBox::warningContinueCancel(this,i18n("<qt>Do you really want to remove <br><b>%1</b><br> from the project?</qt>").arg(nice), i18n("Remove From Project"), KStdGuiItem::remove(), "RemoveFromProject") == KMessageBox::Continue )
     {
       if ( currentKFileTreeViewItem()->isDir() ) url.adjustPath(+1);
       emit removeFromProject(url);
