@@ -21,6 +21,9 @@
 #include <qstringlist.h>
 #include <qdict.h>
 #include <qdir.h>
+#include <qdom.h>
+#include <qfile.h>
+#include <qtextstream.h>
 #include <qwidget.h>
 
 //kde includes
@@ -31,6 +34,7 @@
 #include <kmimetype.h>
 #include <kstandarddirs.h>
 #include <kmessagebox.h>
+#include <kio/netaccess.h>
 #include <klocale.h>
 #include <kprotocolinfo.h>
 #include <kprogress.h>
@@ -661,22 +665,22 @@ bool QuantaCommon::closesTag(QString namespaceName, QString tagName, bool caseSe
         return true;
 }
 
-int QuantaCommon::denyBinaryInsert()
+int QuantaCommon::denyBinaryInsert(QWidget *window)
 {
-  int result = KMessageBox::questionYesNo(0L, i18n("The file type is not recognized. \
+  int result = KMessageBox::questionYesNo(window, i18n("The file type is not recognized. \
   Opening binary files may confuse Quanta.\n Are you sure you want to open this file?"),
-  i18n("Unknown Type"), KStdGuiItem::yes(), KStdGuiItem::no(), "Open Everything");
+  i18n("Unknown Type"), KStdGuiItem::open(), i18n("Do Not Open"), "Open Everything");
  return result;
 }
 
-bool QuantaCommon::checkOverwrite(const KURL& url)
+bool QuantaCommon::checkOverwrite(const KURL& url, QWidget *window)
 {
   bool result = true;
 
-  if (QExtFileInfo::exists(url))
+  if (QExtFileInfo::exists(url, window))
   {
-    if (KMessageBox::warningYesNo(0L,
-            i18n( "<qt>The file <b>%1</b> already exists.<br>Do you want to overwrite it?</qt>" ).arg(url.prettyURL(0, KURL::StripFileProtocol))) == KMessageBox::No)
+    if (KMessageBox::warningContinueCancel(window,
+            i18n( "<qt>The file <b>%1</b> already exists.<br>Do you want to overwrite it?</qt>" ).arg(url.prettyURL(0, KURL::StripFileProtocol)), QString::null, i18n("Overwrite")) == KMessageBox::Cancel)
             result = false;
   }
 
